@@ -13,10 +13,11 @@ import collections
 import numbers
 import warnings
 from redis.exceptions import ResponseError
-from .types import Bulk, ByteString
+from .types import Bulk, String
 from .session import Session
 from .transaction import manipulative, query
 from .warnings import PerformanceWarning
+from . import utils
 
 
 class List(collections.MutableSequence):
@@ -61,7 +62,7 @@ class List(collections.MutableSequence):
     #: (:class:`sider.types.Bulk`) The type of list values.
     value_type = None
 
-    def __init__(self, session, key, value_type=ByteString):
+    def __init__(self, session, key, value_type=String):
         if not isinstance(session, Session):
             raise TypeError('session must be a sider.session.Session '
                             'instance, not ' + repr(session))
@@ -75,13 +76,12 @@ class List(collections.MutableSequence):
         step = 100  # FIXME: it is an arbitarary magic number.
         chunk = None
         decode = self.value_type.decode
-        mark = self.session.mark_query
+        self.session.mark_query([self.key])
         while chunk is None or len(chunk) >= step:
-            mark([self.key])
-            mark = lambda *_: _
-            chunk = self.session.client.lrange(self.key, i, i + step)
+            chunk = self.session.client.lrange(self.key, i, i + step - 1)
             for val in chunk:
                 yield decode(val)
+            i += step
 
     @query
     def __len__(self):
@@ -132,7 +132,7 @@ class List(collections.MutableSequence):
             result = self.session.client.lrange(self.key, start, stop)
             if index.step is not None:
                 result = result[::index.step]
-            return map(decode, result)
+            return list(map(decode, result))
         raise TypeError('indices must be integers, not ' + repr(index))
 
     def __setitem__(self, index, value):
@@ -157,7 +157,7 @@ class List(collections.MutableSequence):
                 raise ValueError('slice with step is not supported for '
                                  'assignment')
             elif index.start in (0, None) and index.stop == 1:
-                seq = map(encode, value)
+                seq = list(map(encode, value))
                 seq.reverse()
                 self.session.mark_manipulative([self.key])
                 if self.session.server_version_info < (2, 4, 0):
@@ -272,7 +272,9 @@ class List(collections.MutableSequence):
         else:
             if not encoded:
                 iterable = (encode(v) for v in iterable)
-            pipe.rpush(self.key, *iterable)
+            n = 100  # FIXME: it is an arbitarary magic number.
+            for chunk in utils.chunk(iterable, n):
+                pipe.rpush(self.key, *chunk)
 
     def insert(self, index, value):
         """Inserts the ``value`` right after the offset ``index``.
