@@ -11,8 +11,9 @@
 from __future__ import absolute_import
 import collections
 from .session import Session
-from .types import Bulk, ByteString
+from .types import Bulk, String
 from .transaction import manipulative, query
+from . import utils
 
 
 class Set(collections.MutableSet):
@@ -61,7 +62,7 @@ class Set(collections.MutableSet):
 
     """
 
-    def __init__(self, session, key, value_type=ByteString):
+    def __init__(self, session, key, value_type=String):
         if not isinstance(session, Session):
             raise TypeError('session must be a sider.session.Session '
                             'instance, not ' + repr(session))
@@ -601,7 +602,8 @@ class Set(collections.MutableSet):
             else:
                 offline_sets.append(operand)
         union = set()
-        for value_type, group in online_sets.iteritems():
+        for value_type in online_sets:
+            group = online_sets[value_type]
             keys = (s.key for s in group)
             self.session.mark_query([self.key])
             subset = self.session.client.sunion(*keys)
@@ -747,7 +749,7 @@ class Set(collections.MutableSet):
         def block(trial, transaction):
             pipe = self.session.client
             if online_sets:
-                keys = (operand.key for operand in online_sets)
+                keys = [operand.key for operand in online_sets]
                 self.session.mark_manipulative()
                 pipe.sunionstore(self.key, self.key, *keys)
             update = self._raw_update
@@ -758,14 +760,15 @@ class Set(collections.MutableSet):
     def _raw_update(self, members, pipe):
         key = self.key
         encode = self.value_type.encode
-        members = [encode(v) for v in members]
-        if members:
-            self.session.mark_manipulative()
-            if self.session.server_version_info < (2, 4, 0):
-                for member in members:
-                    pipe.sadd(key, member)
-            else:
-                pipe.sadd(key, *members)
+        members = (encode(v) for v in members)
+        self.session.mark_manipulative()
+        if self.session.server_version_info < (2, 4, 0):
+            for member in members:
+                pipe.sadd(key, member)
+        else:
+            n = 100  # FIXME: it is an arbitarary magic number.
+            for chunk in utils.chunk(members, n):
+                pipe.sadd(key, *chunk)
 
     def intersection_update(self, *sets):
         """Updates the set with the intersection of itself and
